function [press,cumpress,minlv,b] = plscv1(x,y,lv,noninter)
%  This program calculates the PRESS for the PLS model as each
%  sample is left out of the data set once.  If the data set is
%  big it may take a long time to run since it rebuilds the PLS
%  model once for each sample.  The outputs are a matrix showing
%  how the PRESS changes with each lv for each sample (press),
%  a vector of the cumulative PRESS (cumpress) for all the tests,
%  the number of lvs at minimum press (minlv) and the final
%  vector or matrix b based on minlv or an optional overide value
%  specified at the prompt by the user.  The I/O format is:
%  [press,cumpress,minlv,b] = plscv1(x,y,lv,noninter)
%  Where x is the independent data block, y is the dependent 
%  data block and lv is the maximum number of latent variables
%  to consider.  The last input, noninter, is optional and is
%  used to supress the prompt for user over-ride of the number
%  of latent variables chosen.  Any input here will suppress the prompt.

%  Copyright
%  Barry M. Wise
%  1991

[mx,nx] = size(x);
[my,ny] = size(y);
if mx ~= my
  error('x and y must have the same number of samples.')
end
if lv > nx
  error('lv must <= number of columns in x')
end
%  Initialize matrices
cumpress = zeros(1,lv);
press = zeros(mx,lv);
%  Arrange calibration and test matrices
clc
for i = 1:mx
  home
  s = sprintf('Now working on test set %g out of %g',i,mx);
  disp(s)
  newx = [x(1:i-1,:);[x(i+1:mx,:)]];
  xtest = x(i,:);
  newy = [y(1:i-1,:);[y(i+1:mx,:)]];
  ytest = y(i,:);
%  Obtain new pls model  
  [p,q,w,t,u,b,ssq] = pls(newx,newy,lv);
%  Calculate PRESS and plot
  for j = 1:lv
    ypred = plspred(xtest,b,p,q,w,j);
    press(i,j) = sum(sum((ypred-ytest).^2));
  end 
  plot(press(i,:))
  s = sprintf('Number of LVs vs. PRESS for Sample Number %g',i);
  title(s)
  xlabel('Number of Latent Variables')
	 ylabel('PRESS')
end
pause(2)
%  Calculate cumulative PRESS and plot
cumpress = sum(press);
plot(cumpress)
title('Number of Latent Variable vs. Cumulative PRESS')
xlabel('Number of Latent Variables')
ylabel('Cumulative PRESS')
%  Find and display number of lvs for minimum PRESS
[a,minlv] = min(cumpress);
t = sprintf('Minimum Cumulative PRESS is at %g LVs',minlv);
disp(t)
if nargin == 3
answ = input('Would you like to choose a different number of LVs?  (Yes = 1) ');
if answ == 1
  txt = sprintf('How many Latent Variables would you like? (Max = %g) ',minlv);
  minlv = input(txt);
end
end
disp('  ')
disp('Now working on final PLS model')
[P,Q,W,T,U,bb,ssqdif] = pls(x,y,minlv);
b = conpred1(bb,W,P,Q,minlv);
plot(b), hold on, plot(b,'o'), plot(zeros(b),'-g'), hold off
s = sprintf('Regression Coefficients in Final Model with %g LVs',minlv);
title(s)
xlabel('Variable Number')
ylabel('Regression Coefficient')
